package org.cyberlab;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cyberlab.local.recognition.LocalModel;

import android.app.Activity;


/**
 * 
 * Keeps track of categories, done categories and helps keeping loaded models in memory. 
 * 
 * Implements singleton pattern.
 * @author Mats-Gøran Karlsen
 *
 */
public class MemoryHelper {
	class Model{
		private ArrayList<Long> modelId;
		private ArrayList<LocalModel<?>> modelData;
		
		public Model(List<Long> modelId, List<LocalModel<?>> data) {
			
			//defensive copy
			this.modelId = new ArrayList<Long>();
			this.modelData = new ArrayList<LocalModel<?>>();
			this.modelId.addAll(modelId);
			this.modelData.addAll(data);
		}
		public ArrayList<Long> getIds(){
			return modelId;
		}
		
		public ArrayList<LocalModel<?>> getModelData(){
			return modelData;
		}
	}
	
	private static MemoryHelper instance = null;
	private Map<Long,String> doneCategories;
	private Map<Long, String> categories;
	private Map<String, Model>  loadedModels;
	
	/**
	 * Get an instance of the MemoryHelper
	 * @return
	 */
	public static MemoryHelper getMemoryHelper(){
		if (instance == null) {
			instance = new MemoryHelper();
		}
		return instance;
	}
	
	/**
	 * 
	 * Add quiz categories
	 * @param act
	 * @param categories
	 */
	public void addCategories(InitializeCategoryActivity act, Map<Long, String> categories){
		//Make defensive copy 
        this.categories.putAll(categories);  
	}
	
	/**
	 * 
	 * Add models associated with the activity
	 * Add models 
	 * @param act
	 * @param models
	 */
	public void addModels(Activity act, List<Long> modelId, List<LocalModel<?>> data){
		String key = act.getClass().getCanonicalName();
		loadedModels.put(key, new Model(modelId, data));
	}

	/**
	 * Mark quiz category as done
	 * 
	 * @param id
	 */
	public void categoryDone(long id){
		doneCategories.put(id, categories.get(id));
	}

	/**
	 * 
	 * Check if a given quiz category is done
	 * @param id
	 * @return
	 */
	public boolean checkCategoryDone(long id){
		if (doneCategories.get(id) == null) {
			return false;
		}
		return true;
	}

	/**
	 * 
	 * Gets a list with the name of all available categories
	 * @return
	 */
	public List<String> getAvailableCategories(){
		ArrayList<String> available = new ArrayList<String>();
		for (Map.Entry<Long, String> category : categories.entrySet()) {
			if (!doneCategories.containsKey(category.getKey())) {
				available.add(category.getValue());
			}
		}
		return available;
	}
	
	/**
	 * 
	 * Get all quiz category names and ids
	 * @return
	 */
	public Map<Long, String> getCategories(){
		return categories;
	}

	/**
	 * 
	 * Get the name of a quiz category
	 * @param id
	 * @return
	 */
	public String getCategoryName(long id){
		return categories.get(id);
	}

	/**
	 * 
	 * Get all models previously loaded for a given activity
	 * @param act
	 * @return
	 */
	public Model getModels(Activity act){
		String key = act.getClass().getCanonicalName();
		return loadedModels.get(key);
	}
	
	/**
	 * 
	 * Checks whether the MemoryHelper contains models for an activity
	 * @param act
	 * @return true if the MemoryHelper contains models for the activity
	 */
	public boolean hasModels(Activity act){
		String key = act.getClass().getCanonicalName();
		if (loadedModels.get(key) == null) {
			return false;
		}
		return true;
	}

	/**
	 * 
	 * Remove models associated with the activity
	 * @param act
	 */
	public void removeModels(Activity act){
		String key = act.getClass().getCanonicalName();
		loadedModels.remove(key);
	}
	
	protected MemoryHelper(){
		doneCategories = new HashMap<Long, String>();
		categories = new HashMap<Long, String>();
		loadedModels = new HashMap<String, Model>();		
	}
	
}
