package org.cyberlab;

import org.cyberlab.local.recognition.LocalMatch;
import org.cyberlab.local.recognition.Unknown;

import android.content.Context;
import android.graphics.Bitmap;
import android.util.AttributeSet;
import android.widget.ImageView;

/**
 * Custom ImageView. Rationale: Easy access to original Bitmap set in ImageView. Keeps state of original scaled Bitmap from image resource. 
 * This allows for multiple constructions using the same Bitmap. Unknown is linked to a specific image and is therefore kept inside 
 * RecognitionData. Unknown does not guarantee that its Bitmap is the same as that used for construction (example. could be a scaled version 
 * of the original), and thus should not be reused by the application more than once for construction.
 * 
 * 
 * 
 * Use Case 1:  User decides to build new model but only wants to change one of the Bitmaps. The problem would arise if the user decides to
 * modify the bitmap in ImageView for user feedback (example: indicate quality of model) after each creation. ImageView would then hold an
 * invalid bitmap. 
 * 
 * Use Case 2: Unknown object is to be built on an external thread so UI thread can do user feedback. Since ImageView does not have a
 * convenient getImageBitmap after it is set other than getDrawable, it is convenient to hold on to the Bitmap.
 * 
 * @author Mats-Gøran Karlsen
 *
 */
public class CustomImageView extends ImageView{
	/**
	 * Data relevant to recognition
	 * 
	 */
	class RecognitionData{
		Bitmap bm;
		Unknown obj;
	}
	
	private RecognitionData recD;
	
	public CustomImageView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		recD = new RecognitionData();
	}

	public CustomImageView(Context context, AttributeSet attrs) {
		super(context, attrs);
		recD = new RecognitionData();
	}

	public CustomImageView(Context context) {
		super(context);
		recD = new RecognitionData();
	}

	/**
	 * Draw keypoints from unknown onto ImageView
	 */
	public void drawKeypoints(){
		if (recD != null && recD.obj != null) {
			super.setImageBitmap(recD.obj.drawKeypoints());
		}
	}

	/**
	 * Draws a match bounding box around matched keypoints of an Unknown and a LocalMatch object onto Bitmap stored
	 * inside LocalMatch
	 * 
	 * @param match LocalMatch object
	 */
	public void drawMatchBoundingBox( LocalMatch<?> match ) {
		super.setImageBitmap( match.drawBoundingBox() );
	}

	public Bitmap getImageBitmap(){
		return recD.bm;
	}

	public Unknown getUnknown(){
		return recD.obj;
	}

	public RecognitionData getRecognitionData(){
		return recD;
	}

	@Override
	public void setImageBitmap(Bitmap bm) {
		super.setImageBitmap(bm);
		this.recD.bm = bm;
		this.recD.obj = null;
	}
	
	public void setUnknown(Unknown obj){
		this.recD.obj = obj;
	}
	
	public void setRecocnitionData(RecognitionData recD) {
		this.recD = recD;
		if (recD.obj != null) {
			super.setImageBitmap(recD.obj.drawKeypoints());
		}else if(recD.bm != null){
			super.setImageBitmap(recD.bm);
		}
	}
	
}
