package org.cyberlab;

import java.io.File;
import java.net.Authenticator;
import java.net.MalformedURLException;
import java.net.URL;

import org.cyberlab.R;
import org.cyberlab.local.db.QuizSQLiteDBAdapter;
import org.cyberlab.support.CopyFromURLToHttp;
import org.cyberlab.support.UrlAuthentication;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.Button;
import android.widget.Toast;

/**
 * 
 * First activity that after application launch. Shows a list of options and offers app preferences in options menu.
 * 
 * @author Mats-Gøran Karlsen
 *
 */
public class MainMenuActivity extends Activity {

	private static final String TAG = "MainMenuActivity";
	private static final String URL_PROTOCOL = "http";
	private SharedPreferences prefs;
	private Button set, assignment, object;
	private OnClickListener buttonClickListener = new OnClickListener() {
		@Override
		public void onClick(View arg0) {
			if (!prefs.contains("firstrun")) {
				showDialog(1);
				return;
			}
			switch (arg0.getId()) {
			case R.id.sets:
				startActivity(new Intent(MainMenuActivity.this, SetListActivity.class));
				break;
			case R.id.assignments:
				startActivity(new Intent(MainMenuActivity.this, AssignmentListActivity.class));
				break;
			case R.id.objects:
				startActivity(new Intent(MainMenuActivity.this, ModelListActivity.class));
				break;
			default:
				break;
			}
		}
	};
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.mainmenuoptions, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.prefs:
			Intent intent = new Intent(MainMenuActivity.this, QuizAdminPrefActivity.class);
			startActivity(intent);
			return true;
		case R.id.upload_database:
			uploadDB();
			return true;
		case R.id.download_database:
			if (downloadDB()) {
				if (!prefs.contains("firstrun")) {
					firstrun();
				}
			}
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}		
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.main);
		prefs = PreferenceManager.getDefaultSharedPreferences(this);
		PreferenceManager.setDefaultValues(this, R.xml.preference, false);
		set = (Button) findViewById(R.id.sets);
		set.setOnClickListener(buttonClickListener);
		assignment = (Button) findViewById(R.id.assignments);
		assignment.setOnClickListener(buttonClickListener);
		object = (Button) findViewById(R.id.objects);
		object.setOnClickListener(buttonClickListener);
		Authenticator.setDefault(new UrlAuthentication(prefs.getString("databasehostuser", null),
														prefs.getString("databasehostpass", null)));
		//check if the application has run before
		if (!prefs.contains("firstrun")) {
			if (downloadDB()) {
				firstrun();
			}else{
				showDialog(1);
			}
		}
	}

	@Override
	protected Dialog onCreateDialog(int id) {
		String text = "Please make sure that the SQLite database settings in preferences are correct and " +
				" that the host is reachable by the device. Also make sure that the device has network resources available. When this is fixed manually download" +
				" the database using the options menu.";
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle("Database download failed!")
		.setMessage(text)
		.setCancelable(false)
		.setPositiveButton("I understand", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dismissDialog(1);
			}
		});
		AlertDialog alert = builder.create();
		return alert;
	}
	
	/**
	 * Gets run the first time the application starts or after the first successful database download.
	 * Makes sure the application has a valid database. Rationale for not instantiating a local database is that
	 * nobody who administers a database would modify and upload an outdated version of a database.  
	 */
	private void firstrun(){
		SharedPreferences.Editor editor = prefs.edit();
		editor.putBoolean("firstrun", true);
		editor.commit();
	}

	/**
	 * Download the database from a predefined host defined in default SharedPreferences 
	 */
	private boolean downloadDB(){
		//in order to install the database QuizSQLiteDBAdapter must be instantiated using one of the constructors 
		//that takes either an URL or an InputStream.
		@SuppressWarnings("unused")
		QuizSQLiteDBAdapter dB = null;
		URL fileLocation = null;
		Exception processException = null;
		String text = null;
		try {
			fileLocation = new URL(URL_PROTOCOL, 
					prefs.getString("databasehost", null), 
					Integer.parseInt(prefs.getString("databasehostport", null)), 
					prefs.getString("databasename", null));
			dB = new QuizSQLiteDBAdapter(this, fileLocation);
		} catch (MalformedURLException e) {
			text = "Malformed URL "+fileLocation.toString()+". "+e.getMessage();
			processException = e;
		} catch (Exception e) {
			text = "Error downloading database from "+fileLocation.getHost()+".\n"+e.getMessage();
			processException = e;
		}finally{
			if (processException != null) {		
				Log.e(TAG, text, processException);
				Toast.makeText(this, text, Toast.LENGTH_LONG).show();
				return false;
			}
		}
		text = "New database downloaded";
		Log.v(TAG, "Database downloaded from " +fileLocation.getHost());
		Toast.makeText(this, text, Toast.LENGTH_LONG).show();
		return true;
	}

	/**
	 * Upload the database to a predefined host defined in default SharedPreferences
	 */
	private void uploadDB(){
		QuizSQLiteDBAdapter dB = null;
		URL fileLocation = null;
		Exception processException = null;
		String text = null;
		String filename = prefs.getString("databasename", null);
		int index = filename.lastIndexOf("/");
		filename.substring(index+1);
		File database = new File(getExternalFilesDir(null), filename.substring(index+1));
		dB = new QuizSQLiteDBAdapter(this);
		try {
			dB.open();
			dB.exportDatabase(database);
			fileLocation = new URL(URL_PROTOCOL, 
					prefs.getString("databasehost", null), 
					Integer.parseInt(prefs.getString("databasehostport", null)),
					prefs.getString("databasehostscript", null));
			CopyFromURLToHttp.process(database.toURL(), fileLocation);
		} catch (Exception e) {
			processException = e;
			text = "Error uploading: "+processException.getMessage();
		} finally{
			if (dB.isOpen()) {
				dB.close();
			}
			if (processException != null) {
				Log.e(TAG, text,processException);
			}else{
				text = "Database upload success";
				Log.v(TAG, "Database uploaded as " +database.getName()+" to "+fileLocation.getHost());
			}
			Toast.makeText(this, text, Toast.LENGTH_LONG).show();
			database.delete();
		}
	}
}
